using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

//追加
using AnimatedSprite;

namespace WindowsGame1
{
    /// <summary>
    /// 基底 Game クラスから派生した、ゲームのメイン クラスです。
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        /// <summary>
        /// アニメーションテクスチャ
        /// </summary>
        AnimatedTexture2DSelectFrames numberAnimationTexture;
        
        /// <summary>
        /// アニメーションテクスチャ
        /// </summary>
        AnimatedTexture2DSelectFrames characterAnimation;

        bool UpKeyIsDown = false;
        bool DownKeyIsDown = false;
        bool LeftKeyIsDown = false;
        bool RightKeyIsDown = false;

        Vector2 characterPos;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// ゲームが実行を開始する前に必要な初期化を行います。
        /// ここで、必要なサービスを照会して、関連するグラフィック以外のコンテンツを
        /// 読み込むことができます。base.Initialize を呼び出すと、使用するすべての
        /// コンポーネントが列挙されるとともに、初期化されます。
        /// </summary>
        protected override void Initialize()
        {
            // TODO: ここに初期化ロジックを追加します。
            characterPos = new Vector2(500, 100);
            this.Window.Title = "方向キーでキャラクタが歩きます(斜めには対応していません)";
            base.Initialize();
        }

        /// <summary>
        /// LoadContent はゲームごとに 1 回呼び出され、ここですべてのコンテンツを
        /// 読み込みます。
        /// </summary>
        protected override void LoadContent()
        {
            // 新規の SpriteBatch を作成します。これはテクスチャーの描画に使用できます。
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: this.Content クラスを使用して、ゲームのコンテンツを読み込みます。

            //再生するフレームと順を指定する
            characterAnimation = new AnimatedTexture2DSelectFrames(Content.Load<Texture2D>("CharacterAnimation"), 4, 6, 8, new int[] { 0, 1, 2, 1 });
            //直立絵にしておく
            characterAnimation.SetCurrentFrameNumber(1);

            //再生範囲を指定する
            numberAnimationTexture = new AnimatedTexture2DSelectFrames(Content.Load<Texture2D>("NumberAnimation"), 2, 4, 1, 4, 7); 
        }

        /// <summary>
        /// UnloadContent はゲームごとに 1 回呼び出され、ここですべてのコンテンツを
        /// アンロードします。
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: ここで ContentManager 以外のすべてのコンテンツをアンロードします。
        }

        /// <summary>
        /// ワールドの更新、衝突判定、入力値の取得、オーディオの再生などの
        /// ゲーム ロジックを、実行します。
        /// </summary>
        /// <param name="gameTime">ゲームの瞬間的なタイミング情報</param>
        protected override void Update(GameTime gameTime)
        {
            // ゲームの終了条件をチェックします。
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: ここにゲームのアップデート ロジックを追加します。

            //方向キーで向きと位置を調整
            if (Keyboard.GetState().IsKeyDown(Keys.Down))
            {
                characterPos.Y = characterPos.Y + 1;
                if(!DownKeyIsDown)
                    characterAnimation.SetFrames(new int[] { 0, 1, 2, 1 });
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Left))
            {
                characterPos.X = characterPos.X - 1;
                if (!LeftKeyIsDown)
                    characterAnimation.SetFrames(new int[] { 6, 7, 8, 7 });
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Right))
            {
                characterPos.X = characterPos.X + 1;
                if (!RightKeyIsDown)
                    characterAnimation.SetFrames(new int[] { 12, 13, 14, 13 });
            }
            if (Keyboard.GetState().IsKeyDown(Keys.Up))
            {
                characterPos.Y = characterPos.Y - 1;
                if (!UpKeyIsDown)
                    characterAnimation.SetFrames(new int[] { 18, 19, 20, 19 });
            }

            //キー入力の更新
            DownKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Down);
            LeftKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Left);
            RightKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Right);
            UpKeyIsDown = Keyboard.GetState().IsKeyDown(Keys.Up);

            //方向キーが入っていなかったらアニメーションを止める
            if (!DownKeyIsDown && !LeftKeyIsDown && !RightKeyIsDown && !UpKeyIsDown)
            {
                //表示するフレームを立ち絵に設定する
                characterAnimation.SetCurrentFrameNumber(characterAnimation.GetFrames()[1]);
                characterAnimation.Pause();
            }
            else
                characterAnimation.Play();

            //アニメーションの更新
            numberAnimationTexture.Update((float)gameTime.ElapsedGameTime.TotalSeconds);
            characterAnimation.Update((float)gameTime.ElapsedGameTime.TotalSeconds);

            base.Update(gameTime);
        }

        /// <summary>
        /// ゲームが自身を描画するためのメソッドです。
        /// </summary>
        /// <param name="gameTime">ゲームの瞬間的なタイミング情報</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.DarkBlue);

            // TODO: ここに描画コードを追加します。

            spriteBatch.Begin();

            //アニメーションの現在のフレームの描画
            numberAnimationTexture.DrawFrame(spriteBatch, new Vector2(300, 100));
            characterAnimation.DrawFrame(spriteBatch, characterPos);

            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
